// **********************************************************************
//                             Lode Runner
//                        ---------------------
//
//  File        : LevelEditorMode.cpp
//  Description : 
//  Author      : Guillaume Delhumeau <guillaume.delhumeau@gmail.com>
//  License     : GNU General Public License 3
//
// **********************************************************************

#include "LevelEditorMode.h"
#include "Memory.h"

/************************************************************************/
/* Constructor                                                          */
/************************************************************************/
LevelEditorMode::LevelEditorMode():
_selector_menu(_resources)
{
	_p_level = my_new Level();
}

/************************************************************************/
/* Destructor                                                           */
/************************************************************************/
LevelEditorMode::~LevelEditorMode()
{
	my_delete(_p_level);
}

/************************************************************************/
/* Load graphics                                                        */
/************************************************************************/
void LevelEditorMode::load_graphics(CL_GraphicContext & gc)
{
	_resources.load_theme("resources/themes/theme01.zip", gc);
}

/************************************************************************/
/* Draw                                                                 */
/************************************************************************/
void LevelEditorMode::draw(CL_GraphicContext & gc)
{
	// Draw grid
	for(unsigned int c=0; c<=Level::NB_COLS; ++c)
	{
		CL_Draw::line(gc, (float)c*Level::BLOCK_SIZE, 0,
			(float)c*Level::BLOCK_SIZE, 600, CL_Colorf::gray);
	}
	for(unsigned int r=0; r<=Level::NB_ROWS; ++r)
	{
		CL_Draw::line(gc, 0, (float)r*Level::BLOCK_SIZE,
			800, (float)r*Level::BLOCK_SIZE, CL_Colorf::gray);
	}		

	// Draw blocks
	for(unsigned int c=0; c<Level::NB_COLS; ++c)
	{
		for(unsigned int r=0; r<Level::NB_ROWS; ++r)
		{
			Block * p_block = _p_level->get_block(c, r);
			if(p_block)
			{
				p_block->draw(gc);
			}
		}
	}

	_selector_menu.draw(gc);
}

/************************************************************************/
/* Events                                                               */
/************************************************************************/
void LevelEditorMode::events(CL_InputContext & ic, ZoomParameters & zoom)
{
	if(_selector_menu.events(ic, zoom))
	{
		return;
	}
	if(ic.get_mouse_count()>0)
	{
		CL_InputDevice & mouse = ic.get_mouse();

		int mouse_x = mouse.get_x();
		int mouse_y = mouse.get_y();

		int col = (mouse_x-zoom.dx) / (Level::BLOCK_SIZE * zoom.scale);
		int row = (mouse_y-zoom.dy) / (Level::BLOCK_SIZE * zoom.scale);

		if(col >= 0 && col < Level::NB_COLS &&
			mouse_y >= 0 && row < Level::NB_ROWS)
		{
			if(mouse.get_keycode(CL_MOUSE_LEFT))
			{
				Block * p_block = NULL;
				switch(_selector_menu.get_selection())
				{
				case SelectorMenu::SELECTION_WALL:
					p_block = my_new WallBlock();
					break;
				case SelectorMenu::SELECTION_WALL_2:
					p_block = my_new WallBlock2();
					break;
				case SelectorMenu::SELECTION_SCALE:
					p_block = my_new ScaleBlock();
					break;
				case SelectorMenu::SELECTION_ROPE:
					p_block = my_new RopeBlock();
					break;
				case SelectorMenu::SELECTION_BREAKABLE:
					p_block = my_new BreakableWallBlock();
					break;
				case SelectorMenu::SELECTION_LOOT:
					p_block = my_new LootBlock();
					break;
				default:
					break;
				}
				if(p_block)
				{
					p_block->set_resources(_resources);
					p_block->set_position(col * Level::BLOCK_SIZE,
						row * Level::BLOCK_SIZE);
						
					Block * p_old_block = _p_level->get_block(col, row);
					if(p_old_block)
					{
						my_delete(p_old_block);
					}

					_p_level->set_block(col, row, p_block);
				}
			}
			else if(mouse.get_keycode(CL_MOUSE_RIGHT))
			{
				Block * p_block = _p_level->get_block(col, row);
				if(p_block)
				{
					my_delete(p_block);
					_p_level->set_block(col, row, NULL);
				}			
			}
		}
	}
}

/************************************************************************/
/* Update                                                               */
/************************************************************************/
void LevelEditorMode::update(float delta_time)
{
	_selector_menu.update(delta_time);
}
